
#ifndef __JBT_H
#define __JBT_H

#define STRICT
//#define SCRIPTSUPPORT
#include "orbitersdk.h"
#include <vector>
#include <math.h>
#include "UmmuSDK.h"
#include "OrbiterSoundSDK40.h"

// ==============================================================
// Some vessel parameters
// ==============================================================
const double  B1200_SIZE       = 8.55;          // mean radius [m]
const double  B1200_LENGTH	   = 8.55;			//Length in m
const double  B1200_WIDTH	   = 2.9;			//Width in m
const double  B1200_HEIGHT	   = 2.5;				//Height in m
const double  B1200_COG		   = -0.01;				//Centre of Gravity in m

const VECTOR3 B1200_CS         = {12.3,22.2,5.97}; // x,y,z cross sections [m^2]
const VECTOR3 B1200_TOWBARI	   = {0.25,1,0.25};

const VECTOR3 B1200_PMI        = {6.3,5.85,1.9};
//const VECTOR3 B1200_RD         = {0.025,0.025,0.02};//{0.05,0.1,0.05};  // rotation drag coefficients
const double  B1200_EMPTYMASS  = 54400;			// empty vessel mass [kg]
const double  B1200_FUELMASS   = 176.8;			// max fuel mass [kg]
const double  B1200_FUELTIME   = 78.7;			//Time until fuel tank is empty in Hours
const double  B1200_FUELCOMSUMP= 0.0000024666;//mass per rpm per sec
const double  B1200_TOWBARMASS  = 100;			// Towbar mass [kg]

const double  B1200_TYREDIA	   = 1.5;				//Diameter of the tyre in m
const double  B1200_TYREWIDTH  = 0.77;				//Width of the tyre in m
const double  B1200_WHEELBASE  = 3.66;				//Wheelbase in m
const double  B1200_TURNINGCIRCLE_2WS = 12.1;			//Outside Turning Circle in m
const double  B1200_WHEELOFFSET = (B1200_WIDTH - B1200_TYREWIDTH)/2; //X offset of each wheel
const double  B1200_TURNINGANGLEOUTSIDE_F = asin(B1200_WHEELBASE/B1200_TURNINGCIRCLE_2WS);	//Outside Turning Angle in rads Front Wheels
const double  B1200_TURNINGANGLEINSIDE_F = asin(B1200_WHEELBASE/(B1200_TURNINGCIRCLE_2WS-2*B1200_WHEELOFFSET));	//Inside Turning Angle in rads Front Wheels
const double  B1200_TURNINGANGLE_2WS = asin(B1200_WHEELBASE/(B1200_TURNINGCIRCLE_2WS-B1200_WHEELOFFSET));	//Mean Turning Angle in rads

//const double  B1200_WHEELBASE_4WS = B1200_WHEELBASE/2;
const double  B1200_TURNINGCIRCLE_4WS = 7.1;			//Outside Turning Circle in m
const double  B1200_TURNINGANGLEOUTSIDE_R = asin(B1200_WHEELBASE/B1200_TURNINGCIRCLE_4WS)-B1200_TURNINGANGLEOUTSIDE_F;	//Outside Turning Angle in rads Rear Wheels
const double  B1200_TURNINGANGLEINSIDE_R = asin(B1200_WHEELBASE/(B1200_TURNINGCIRCLE_4WS-2*B1200_WHEELOFFSET))-B1200_TURNINGANGLEINSIDE_F;	//Inside Turning Angle in rads Rear Wheels
const double  B1200_TURNINGANGLE_4WS = asin(B1200_WHEELBASE/(B1200_TURNINGCIRCLE_4WS-B1200_WHEELOFFSET));	//Mean Turning Angle in rads

const double B1200_TURNINGANGLE_CRAB = B1200_TURNINGANGLEOUTSIDE_R>B1200_TURNINGANGLEOUTSIDE_F?B1200_TURNINGANGLEOUTSIDE_R:B1200_TURNINGANGLEOUTSIDE_F; //Crab angle is the smallest steering angle

const double  B1200_MAXSPEED   = 6.67;				//Forward Speed in m/s
const double  B1200_MINSPEED   = 2.78;				//Reverse Speed in m/s
const double  B1200_BRAKEFORCE = 200000;				//Brake power in N

const double  B1200_DRAWBARPULL = 320000;			//Drawbar Pull in N
//const double  B1200_DRAWBARPULL = 3200000;			//Drawbar Pull in N
const double  B1200_TOWBARANGLEMAX = 180;			//The maximum angle the tow bar can move in degrees
const double  B1200_NOSEANGLEMAX = 180;				//The maximum angle the nose wheel can move in degrees

const double  B1200_TOWBARLENGTH  = 6.5;			// Towbar length [m]
const double  MAX_GRAPPLING_DIST  = 10;			// Max Distance away from vessel
//const double TOWBARANGLE = -0.015623728;			//tan^-1((0.5-0.45)/3.2) 0.5 is height of towpoint from bottom of wheels, 0.45 is the height of towball from bottom of wheels, 3.2 is the distance between towball and wheels of towbar
const double TOWBARANGLE = 0;

const VECTOR3 B1200_TOWBARATTACHFRONT_POS   = {0,0.45,5};      // Front Towbar location [m]
const VECTOR3 B1200_TOWBARATTACHFRONT_DIR   = {0,0,1};         // Front Towbar approach direction
const VECTOR3 B1200_TOWBARATTACHFRONT_ROT   = {0,1,0};        // Front Towbar alignment direction

const VECTOR3 B1200_TOWBARATTACHFRONT_START_POS   = {0,0.45,10.01};      // Front Start Towbar location [m]
const VECTOR3 B1200_TOWBARATTACHFRONT_START_DIR   = {0,0,1};         // Front Start Towbar approach direction
const VECTOR3 B1200_TOWBARATTACHFRONT_START_ROT   = {0,1,0};        // Front Start Towbar alignment direction

//const VECTOR3 B1200_TOWBARATTACHBACK_POS   = {0,1.3,-1};      // Back Towbar location [m]
const VECTOR3 B1200_TOWBARATTACHBACK_POS   = {0,0.45,-3.41};      // Back Towbar location [m]
const VECTOR3 B1200_TOWBARATTACHBACK_DIR   = {0,0,-1};         // Back Towbar approach direction
const VECTOR3 B1200_TOWBARATTACHBACK_ROT   = {0,1,0};        // Back Towbar alignment direction

const VECTOR3 B1200_TOWBARATTACHBACK_START_POS   = {0,0.45,-9.91};      // Back Start Towbar location [m]
const VECTOR3 B1200_TOWBARATTACHBACK_START_DIR   = {0,0,-1};         // Back Start Towbar approach direction
const VECTOR3 B1200_TOWBARATTACHBACK_START_ROT   = {0,1,0};        // Back Start Towbar alignment direction

const double B1200_GEARS[] = {4.5,2.5,2,1.5,1,0.678};
const double B1200_DIFF = 6;
const double B1200_MULTIPLIER = 3;

const double  B1200_ISP        = (B1200_DRAWBARPULL/(B1200_FUELMASS/(B1200_FUELTIME*3600)));             // fuel-specific impulse [m/s]
//const VECTOR3 B1200_TDP[3]     = {{0,B1200_COG,1.83},{-(B1200_WIDTH+B1200_TYREWIDTH)/2,B1200_COG,-1.83},{(B1200_WIDTH+B1200_TYREWIDTH)/2,B1200_COG,-1.83}}; // touchdown points [m]
const VECTOR3 B1200_TDP[3]     = {{0,B1200_COG,5},{-5,B1200_COG,-5},{5,B1200_COG,-5}}; // touchdown points [m]

const DWORD	  B1200_HUDCOLOURS[4] = {0x6AFF61,0xCC0000,0x000099,0x66CCFF};

// ==============================================================
// JBT_B1200 class interface
// ==============================================================

class JBT_B1200: public VESSEL3 {

public:
	JBT_B1200 (OBJHANDLE hVessel, int flightmodel);
	~JBT_B1200 ();
	void clbkSetClassCaps (FILEHANDLE cfg);
	void JBT_B1200::clbkPreStep (double simt, double simdt, double mjd);
	int JBT_B1200::clbkConsumeBufferedKey(DWORD key,bool down,char * kstate);
	void clbkPostCreation(void);
	void clbkLoadStateEx (FILEHANDLE scn, void *vs);
	void clbkSaveState (FILEHANDLE scn);
	//bool clbkDrawHUD  ( int  mode, const HUDPAINTSPEC *  hps, oapi::Sketchpad *  skp);
	//void clbkRenderHUD  ( int  mode,const HUDPAINTSPEC *  hps, SURFHANDLE  hDefaultTex){} 
	//bool clbkLoadPanel2D (int id, PANELHANDLE  hPanel, DWORD  viewW,  DWORD  viewH){return true;}
	bool clbkLoadGenericCockpit(){return false;} //Causes CTDs as the sim is expecting the generic panel and it is not there.
	int  clbkGeneric (int msgid, int prm, void *context);

	bool clbkLoadVC (int id);
	bool clbkVCMouseEvent (int id, int event, VECTOR3 &p);
	bool clbkVCRedrawEvent (int id, int event, SURFHANDLE surf);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);

	/////////////////////////////////////////////////////////////////////
	// 2-ORBITERSOUND EXAMPLE - STORE YOUR UNIQUE ID IN YOUR VESSEL CLASSYou shou
	/////////////////////////////////////////////////////////////////////
	int MyID;	// this is your unique Id, it will identify your ship in OrbiterSound.
	
	//SURFHANDLE exhaust_tex;
	
	//double exhaustStream;

	//PARTICLESTREAMSPEC exhaust_prt;

	// UMMU 2.0 DECLARATION
	UMMUCREWMANAGMENT Crew;
	int iSelectedUmmuMember; // for the SDK demo, select the member to eva
	// The HUD display method variable
	char cUmmuHudDisplay[255]; // UMmu hud char variable
	double dHudMessageDelay; // UMmu hud display delay
	char *SendHudMessage(void); // UMmu hud display function
	// The Add  Mmu without scenery editor declaration see PDF doc
	char cAddUMmuToVessel[255];
	void AddUMmuToVessel(BOOL bStartAdding=FALSE);

	//Lua
	double currentSpeed;
	int transmission;
	int mode;
	bool starter;
	bool parkbrake;
	double fuel; //mass

	ATTACHMENTHANDLE front,back,fronttow,backtow;
	

	//bool indicators[3];//0 is Headlight, 1 is Warning Light, 2 is Park Brake. True is on, False is off
	bool headlights;
	bool headLights();
	bool warninglights;
	bool warningLights();
	bool parklights;
	bool parkLights();
	bool brakelights;
	bool brakeLights();

	bool AttachPlane (bool frontPoint,bool Attach);
	bool AttachTowbar (bool frontPoint, bool Attach);

private:
	
	UINT anim_towbar,anim_LeftFront,anim_RightFront,anim_LeftRear,anim_RightRear,anim_LeftFrontSteer,anim_RightFrontSteer,anim_LeftRearSteer,anim_RightRearSteer;
	double anim_LeftFront_state,anim_RightFront_state,anim_LeftRear_state,anim_RightRear_state,anim_LeftFrontSteer_state,anim_RightFrontSteer_state,anim_LeftRearSteer_state,anim_RightRearSteer_state;
	
	MGROUP_ROTATE* LeftFront,* RightFront,* LeftRear,* RightRear;

	//VC
	UINT anim_Speedo,anim_Rev,anim_Fuel,anim_Gear;
	UINT anim_Brake, anim_Throttle, anim_Steer, anim_Park, anim_GearStick;
	UINT anim_Head, anim_Warn, anim_Complex, anim_SteerMode;
	UINT anim_Button[4], anim_FuelBtn;


	BEACONLIGHTSPEC warninglights_beac[2], parklights_beac[2], indicatorlights_beac[4], reverselights_beac[2], headlights_beac[2],brakelights_beac[2],illumlights_beac[2];
	LightEmitter* headlight;

	//Lengths
	double lb0,lf1,lb1,lf2;
	double lb0a,lf1a;
	//Angles
	double theta1, theta2;

	double towbarElevation;

	void ChangeAttachment(ATTACHMENTHANDLE attachmentToChange,double elevation, double azimuth, int towbar);
	
	THRUSTER_HANDLE th_Input, dummy;
	PROPELLANT_HANDLE fuelDummy;

	double timeIdle;

	//Int to define the flight realism level
	//Attach is for Action areas to set
	int complex;

	//Attach is for Action areas to set
	bool attachAction;

	//New Movement Code
	
	bool lastLatLongSet;
	double lastLat;
	double lastLong;
	double lastHead;
	VESSELSTATUS2 vs;
	OBJHANDLE hEarth;

	//New engine
	int gear;

	int steerMode;

	//double rpm; //rpm
	bool clutch;//true -> closed false -> open
	double engineRPM;
	

	// script interface-related methods, implemented in JBT_Lua.cpp
	int Lua_InitInterpreter (void *context);
	int Lua_InitInstance (void *context);

	struct TowbarYoke{
		char tags[8];
		double width;
		double length;
	};

	std::vector<TowbarYoke> towbarData;

	//VC
	void RedrawSpeedo();
	void RedrawRev();
	void RedrawFuel();
	void RedrawGear();
	
	bool RedrawText(SURFHANDLE surf);

	void RedrawParkbrake();
	void RedrawSteeringWheel();
	void RedrawThrottle();
	void RedrawBrake();
	void RedrawGearStick();
	void RedrawButtons();
	void RedrawSwitches();
	void RedrawButtons(int id,int event);
	void ButtonLogic(int id);

	//bool RedrawInd(SURFHANDLE surf);
	
	//VISHANDLE visual;                            // handle to DG visual representation
	MESHHANDLE vcmesh;          // VC mesh: global template
	DEVMESHHANDLE vcmesh_dev;          // VC mesh: global template

	void SetMeshGroupVisible(DEVMESHHANDLE hMesh, DWORD dwMeshGroup, bool isVisible);
	
};	

//VC
//Displays
#define VC_DIALS	1
#define	VC_TEXT		3


//Sounds
#define IDLE 1
#define LOW 2
#define MEDIUM 3
#define HIGH 4
#define STARTER 5
#define GROUND 6
#define PARKBRAKES 7
#define CLANG 8
#define SCREACH 9
#define BLINK 10

//Controls
#define	VC_CONTROLS	4
#define	VC_PARK		5
#define	VC_GEAR		6


//Buttons
#define	VC_HEADLIGHT		7
#define	VC_WARNINGLIGHT		8
#define	VC_COMPLEX			9
#define	VC_FUEL		10
#define	VC_BUTTON1	11
#define	VC_BUTTON2	12
#define	VC_BUTTON3	13
#define	VC_BUTTON4	14
#define	VC_STEERMODE	15
#define UMMU_LEFT	16
#define UMMU_RIGHT	17

#endif // !__JBT_H

