// ==============================================================
//                    ORBITER MODULE: HST
//                  Part of the ORBITER SDK
//          Copyright (C) 2001-2007 Martin Schweiger
//                   All rights reserved
//
// JBT_B1299_Lua.cpp
// Script extensions to JBT module
// ==============================================================

#include "JBT_B1200.h"

#ifdef SCRIPTSUPPORT

extern "C" {
#include <lua\lua.h>
#include <lua\lualib.h>
#include <lua\lauxlib.h>
}

// ==========================================================================
// API function prototypes

JBT_B1200 *lua_toJBT (lua_State *L, int idx = 1);
int jbtSpeed (lua_State *L);
int jbtMode (lua_State *L);
int jbtTrans (lua_State *L);
int jbtStarter(lua_State *L);
int jbtGetFuel (lua_State *L);
int jbtRefuel (lua_State *L);
int jbtLights (lua_State *L);
int jbtAttachIndex (lua_State *L);
int jbtAttach (lua_State *L);
int jbtDetach (lua_State *L);

// ==========================================================================
// API initialisation

int JBT_B1200::Lua_InitInterpreter(void *context)
{
	lua_State *L = (lua_State*)context;

	// add interpreter initialisation here

	return 0;
}

int JBT_B1200::Lua_InitInstance(void *context)
{
	lua_State *L = (lua_State*)context;

	// check if interpreter has DG table loaded already
	luaL_getmetatable (L, "VESSEL.JBT_B1200");

	if (lua_isnil (L, -1)) { // register new functions
		lua_pop (L, 1);
		static const struct luaL_reg jbtLib[] = {
			{"speed", jbtSpeed},
			{"mode", jbtMode},
			{"transmission", jbtTrans},
			{"starter", jbtStarter},
			{"getFuel", jbtGetFuel},
			{"refuel", jbtRefuel},
			{"lights", jbtLights},
			{"attachIndex", jbtAttachIndex},
			{"attach", jbtAttach},
			{"detach", jbtDetach},
			{NULL, NULL}
		};

		// create metatable for vessel userdata
		luaL_newmetatable (L, "JBT_B1200.vtable");

		// create a table for the overloaded methods
		luaL_openlib (L, "JBT_B1200.method", jbtLib, 0);

		// create metatable for accessing inherited methods from VESSEL
		luaL_newmetatable (L, "JBT_B1200.base");
		lua_pushstring (L, "__index");
		luaL_getmetatable (L, "VESSEL.vtable");
		lua_settable (L, -3);

		// set HST.base as metatable for JBT.method
		lua_setmetatable (L, -2);

		// point vessel userdata to JBT.method
		lua_pushstring (L, "__index");
		lua_pushvalue (L, -2);
		lua_settable (L, -4);

		// pop HST.method from the stack
		lua_pop (L, 1);
	}

	lua_setmetatable (L, -2);

	return 0;
}

// ==========================================================================
// Script API functions

JBT_B1200 *lua_toJBT (lua_State *L, int idx)
{
	VESSEL **pv = (VESSEL**)lua_touserdata (L, idx);
	JBT_B1200 *jbt = (JBT_B1200*)*pv;
	return jbt;
}

int jbtSpeed (lua_State *L) {
	JBT_B1200 *v = lua_toJBT (L,1);
	double temp = v->currentSpeed;
	lua_pushnumber(L, temp);
	return 1;
}

int jbtMode (lua_State *L){
	JBT_B1200 *v = lua_toJBT (L,1);
	double temp = v->mode;
	lua_pushnumber(L, temp);
	return 1;
}

int jbtTrans (lua_State *L){
	JBT_B1200 *v = lua_toJBT (L,1);
	int action = lua_tointeger (L, 2);
	if (v && action >= -1 && action <= 2)
		v->transmission = action;
	return 0;
}

int jbtStarter(lua_State *L) {
	JBT_B1200 *v = lua_toJBT (L,1);
	int action = lua_tointeger (L, 2);
	if (v && action >= 0 && action < 2)
		v->starter = action == 1? true : false;
	return 0;
}

int jbtGetFuel (lua_State *L){
	JBT_B1200 *v = lua_toJBT (L,1);
	double temp = v->fuel;
	lua_pushnumber(L, temp);
	return 1;
}

int jbtRefuel (lua_State *L){
	JBT_B1200 *v = lua_toJBT (L,1);
	 v->fuel = B1200_FUELMASS;
	return 0;
}

int jbtLights (lua_State *L){
	JBT_B1200 *v = lua_toJBT (L,1);
	int action = lua_tointeger (L, 2);
	if (v && action >= 0 && action < 2){
		if(action == 1){
			v->headlights = false;
			v->headLights();
		}
		else {
			v->headlights = true;
			v->headLights();
		}
	}

	action = lua_tointeger (L, 3);
	if (v && action >= 0 && action < 2){
		if(action == 1){
			v->warninglights = false;
			v->warningLights();
		}
		else {
			v->warninglights = true;
			v->warningLights();
		}
	}

	action = lua_tointeger (L, 4);
	if (v && action >= 0 && action < 2){
		if(action == 1){
			v->parkbrake=true;
		}
		else {
			v->parkbrake=false;
		} 
	}
	return 0;
}

int jbtAttachIndex (lua_State *L){
	JBT_B1200 *v = lua_toJBT (L,1);
	int action = lua_tointeger (L, 2);
	int temp = -1;
	switch(action){
		case -2 :	temp = v->GetAttachmentIndex(v->front);
					break;
		case -1 :	temp = v->GetAttachmentIndex(v->fronttow);
					break;
		case 1 :	temp = v->GetAttachmentIndex(v->backtow);
					break;
		case 2 :	temp = v->GetAttachmentIndex(v->back);
					break;
	}
	lua_pushnumber(L, temp);
	return 1;
}

int jbtAttach (lua_State *L){
	JBT_B1200 *v = lua_toJBT (L,1);
	int action = lua_tointeger (L, 2);
	int temp = 0;
	bool success = false;
	switch(action){
		case -2 :	success = v->AttachPlane(false,true);
					if(success)
						v->mode = -2;
					break;
		case -1 :	success = v->AttachTowbar(false,true);
					if(success)
						v->mode = -1;
					break;
		case 1 :	success = v->AttachTowbar(true,true);
					if(success)
						v->mode = 1;
					break;
		case 2 :	success = v->AttachPlane(true,true);
					if(success)
						v->mode = 2;
					break;
	}
	temp = success?1:0;
	lua_pushnumber(L, temp);
	return 1;
}

int jbtDetach (lua_State *L){
	JBT_B1200 *v = lua_toJBT (L,1);
	int action = lua_tointeger (L, 2);
	int temp = 0;
	bool success = false;
	switch(action){
		case -2 :	success = v->AttachPlane(false,false);
					if(success)
						v->mode = -1;
					break;
		case -1 :	success = v->AttachTowbar(false,false);
					if(success)
						v->mode = 0;
					break;
		case 1 :	success = v->AttachTowbar(true,false);
					if(success)
						v->mode = 0;
					break;
		case 2 :	success = v->AttachPlane(true,false);
					if(success)
						v->mode = 1;
					break;
	}
	temp = success?1:0;
	lua_pushnumber(L, temp);
	return 1;

}


// ==========================================================================

#else // skip script support: dummy stubs

int JBT_B1200::Lua_InitInterpreter(void *context)
{
	return 0;
}

int JBT_B1200::Lua_InitInstance (void *context)
{
	return 0;
}

#endif // SCRIPTSUPPORT